import effects
from util import *
import sound


class BeamWeapon(Object):
   alpha = .25
   firing = False
   readyToPlaySound = True
   projectiles = []
   projectileSpeed = 999
   
   def __init__(self, ship, slow=False):
      self.ship = ship
      self.shotNoiseLength = self.shotNoise.get_length()
      self.shotNoiseTimer = timer.Timer(self.shotNoiseLength*1000)
   
   def draw(self):
      if self.firing:
         self.firing = False
         Object.draw(self)
      else:
         self.readyToPlaySound = True
         self.shotNoise.stop()
      
   def drawModel(self):
      w = sqrt(self.beams)
      l = self.length()
      glNormal(0, 0, 1)
      glBegin(GL_QUADS)
      glVertex(-w, 0, 0)
      glVertex(w, 0, 0)
      glVertex(w, l, 0)
      glVertex(-w, l, 0)
      glEnd()
      
   
   def getCollision(self, ships):
      if not self.firing:
         return
      mindist = self.length
      hitsomething = False
      for ship in ships:
         if not ship.alive or self in ship.weapons:
            continue
         col = isCollisionLineSphere(self.pos, self.end, ship.pos, ship.radius)
         if col:
            hitsomething = True
            dst = dist(self.pos, col)
            if dst < mindist:
               mindist = dst
               mincol = col
               ourship = ship
      if hitsomething:
         dmg = self.damage * timer.percTicked #* cDiff(self.color, ourship.color) 
         ourship.damage(dmg)
         effects.explosion.explodeParticles(sqrt(dmg)*100, mincol, Vector(0, 0, 0), 0, self.color)
         self.end = self.pos + Vectorp(mindist, self.orientation)
  
   def fire(self, pos, angle, targets):
      self.pos = pos
      l = self.deflength()
      self.end = self.ship.pos + Vector([l*cos(angle), l*sin(angle), 0])
      self.orientation = angle
      self.color = ~self.ship.color
      self.firing = True
      if self.readyToPlaySound or self.shotNoiseTimer.ready():
         self.readyToPlaySound = False
         self.shotNoise.play()
  
   def length(self):
      return dist(self.pos, self.end)
 
   def deflength(self):
      w, h, z = display.screenSize
      return sqrt(4*(w**2 + h**2))
 

class spectrumBeamWeapon(BeamWeapon):
   def drawModel(self):
      w = self.width*3
      l = self.length()
      glNormal(0, 0, 1)
#      i = l/100
      i = 0
      chunkLength = 3
      glBegin(GL_QUADS)
      while(i<l):
#         glVertex(-w*uniform(0,4), i-l/100, 0)
#         glVertex(w*uniform(0,4), i-l/100, 0)
         glVertex(-uniform(0, w), i-chunkLength, 0)
         glVertex(uniform(0, w), i-chunkLength, 0)
         glVertex(uniform(0, w), i, 0)
         glVertex(-uniform(0, w), i, 0)        
#         i+=l/100
         i+= chunkLength
      glEnd()

class lightningBeamWeapon(BeamWeapon):
#   ltex = loadTexture("lightning")
   def drawModel(self):
      w = .5
      allw = sqrt(self.beams)
      l = self.length()
      glNormal(0, 0, 1)
#      glEnable(GL_TEXTURE_2D)
#      glBindTexture(GL_TEXTURE_2D, self.ltex)
#      glBlendFunc(GL_SRC_ALPHA, GL_ONE)
      glBegin(GL_QUADS)
      for z in range(-self.beams/2+1, self.beams/2+1):
         oldy = 0
         y = 0
         oldrand = 0
         while True:
            y += uniform(5, 15)
            if y > l:
               break
            currentrand = uniform(-allw, allw)
#            glTexCoord2f(0, 0); glVertex(-w+oldrand, oldy, z)
#            glTexCoord2f(1, 0); glVertex(w+oldrand, oldy, z)
#            glTexCoord2f(1, 1); glVertex(w+currentrand, y, z)
#            glTexCoord2f(0, 1); glVertex(-w+currentrand, y, z)
            glVertex(-w+oldrand, oldy, z)
            glVertex(w+oldrand, oldy, z)
            glVertex(w+currentrand, y, z)
            glVertex(-w+currentrand, y, z)
            oldrand = currentrand
            oldy = y
         glVertex(-w+oldrand, oldy, z)
         glVertex(w+oldrand, oldy, z)
         glVertex(w, l, z)
         glVertex(-w, l, z)
      glEnd()
#      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
#      glDisable(GL_TEXTURE_2D)
      


class Lasergun(lightningBeamWeapon):
   shotNoise = sound.getSound('electricalcurrent')
   type = 'center'
   level = 0
   beams = 3
   damage = .5

class WeakLasergun(lightningBeamWeapon):
   shotNoise = sound.getSound('longlaser1')
   level = -1
   beams = 2
   damage = .1

class BigLasergun(Lasergun):
   shotNoise = sound.getSound('electricalcurrent2')
   level = 1
   beams = 5
   damage = .7

class GiantLasergun(Lasergun):
   shotNoise = sound.getSound('beam01')
   level = 2
   beams = 7
   damage = 1

class InsaneLasergun(GiantLasergun):
   level = 3
   beams = 10
   damage = 1.3

playerWeapons = [Lasergun, BigLasergun, GiantLasergun, InsaneLasergun]


# vim: set et sts=3 sw=3:
