import sys

try:
   from OpenGL.GL import *
   from OpenGL.GLUT import *
   from OpenGL.GLU import *
except ImportError:
   print 'pyOpenGL not installed! http://pyopengl.sourceforge.net/'

import pygame.display
from pygame.locals import *
from vector import *
import options
import timer

class Display:
   blur = False
   postUpdateScreenSize = False
   screenSize = Vector(0,0,0)
   screenResolution = options.resolution
   startCamera = Vector(0, 0, 100)
   toZoom = 0
   zoomInc = .06
   cameraLookAt = Vector(0,0,0)
   cameraUp = Vector(0,1,0)
   
   def __init__(self):
      self.reshape(*self.screenResolution)
      pygame.display.set_caption('Triplex Invaders')
      self.glinit()
      self.resetCamera()
   
   def resetCamera(self):
      self.cameraPos = self.startCamera.copy
      self.clearAll()
      self.objectCamera()
      self.screenSize = self.getScreenSize()
   
   def reshape(self, w, h):
      pygame.display.set_mode((w,h), OPENGL|DOUBLEBUF|RESIZABLE)
      self.screenResolution = (w,h)
      glViewport(0, 0, w, h)
      glMatrixMode(GL_PROJECTION)
      glLoadIdentity()
      gluPerspective(45.0, float(w)/float(h), 1, 500)
      glMatrixMode(GL_MODELVIEW)

   def glinit(self):
      glutInit(sys.argv)
      glClearColor(0.0, 0.0, 0.0, 0.0)
      #glClearColor(0.0, 0.02, 0.18, 0.0)
      glEnable(GL_DEPTH_TEST)
      glShadeModel(GL_SMOOTH)     
      glEnable(GL_NORMALIZE)
      glEnable(GL_LIGHTING)
#      glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, True)
      glEnable(GL_BLEND)
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
#      glEnable(GL_CULL_FACE)
#      glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, GL_BLEND)
   
   def invadersLightInit(self):
      glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE)
      glEnable(GL_COLOR_MATERIAL)
      glLightfv(GL_LIGHT0, GL_POSITION, (0, 0, 50, 0))
      glLightfv(GL_LIGHT0, GL_AMBIENT, (.8, .8, .8, 1))
      glLightfv(GL_LIGHT0, GL_SPECULAR, (0.9, 0.9, 0.9, 1.0))
      glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.8, 0.8, 0.8, 1.0))
      glEnable(GL_LIGHT0)
   
   def gravityLightInit(self):
      glColorMaterial(GL_FRONT_AND_BACK, GL_SPECULAR)
      glEnable(GL_COLOR_MATERIAL)
      glLightfv(GL_LIGHT0, GL_POSITION, (0, 0, 0, 0))
      glLightfv(GL_LIGHT0, GL_AMBIENT, (0, 0, 0, 1))
#      glLightfv(GL_LIGHT0, GL_SPECULAR, (0.9, 0.9, 0.9, 1.0))
      glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.4, 0.4, 0.4, 1.0))
      glEnable(GL_LIGHT0)
   
   def update(self):
      if timer.tenTurnTimer.ready():
         self.zoomCamera(.05*timer.percTicked)
   
   def onScreen(self, vector):
      x, y, z = vector
      w, h, d = self.screenSize
      w, h = w+10, h+10
      a, b, c = self.cameraLookAt
      return (a-w < x < a+w) and (b-h < y < b+h) and abs(z)<50

   def setCameraZ(self, z):
      self.cameraPos.z = z
      self.resetPerspective()
   
   def zoomCamera(self, a):
      self.cameraPos.z += a
      self.resetPerspective()
      
   def resetPerspective(self):
      glMatrixMode(GL_PROJECTION)
      glLoadIdentity()
      w, h = self.screenResolution
      gluPerspective(45.0, float(w)/h, 10, self.cameraPos.z+50)
      glMatrixMode(GL_MODELVIEW)
   
   def getScreenSize(self):
      x, y, z = gluProject(0, 0, 0)
      w, h = self.screenResolution
      topRight = Vector(gluUnProject(w, h, z))
      bottomLeft = Vector(gluUnProject(0, 0, z))
      w = (topRight.x - bottomLeft.x) / 2.0
      h = (topRight.y - bottomLeft.y) / 2.0
      return Vector(w, h, 0)
   
   def clearAll(self):
      if self.blur:
         glClear(GL_DEPTH_BUFFER_BIT)
      else:
         glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
   
   def wordCamera(self):
      glLoadIdentity()
      gluLookAt(0, 0, 100, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)
      
   def hudCamera(self):
      x, y, z = self.cameraPos
      a, b, c = self.cameraLookAt
      p, q, r = self.cameraUp
      glLoadIdentity()
      gluLookAt(x, y, 100, a, b, c, p, q, r)
      
   def objectCamera(self):
      x, y, z = self.cameraPos
      a, b, c = self.cameraLookAt
      p, q, r = self.cameraUp
      glLoadIdentity()
      gluLookAt(x, y, z, a, b, c, p, q, r)
   
   def fade(self):
      glColor(0-0.1, 0.02-0.1, 0.18-0.1, 0.39) #colours were at -1, blur was at 0.1
      glNormal(0, 0, 1)
      x, y, z = self.screenSize
      glBegin(GL_QUADS)
      glVertex(x, y, 9)
      glVertex(-x, y, 9)
      glVertex(-x, -y, 9)
      glVertex(x, -y, 9)
      glEnd()

display = Display()


# vim: set et sts=3 sw=3:
