from ship import *
import powerups
import weapon
from util import *
import effects
import player
import options

if options.difficulty is 'easy':
   powerupFreq = 0.5
elif options.difficulty is 'medium':
   powerupFreq = 0.4
elif options.difficulty is 'hard':
   powerupFreq = 0.3

class Enemy(Ship, AI):
   fireAngle = 0.5
   preEffects = [tilt, effects.Thruster()]
   
   def __init__(self, pos, color):
      self.pos = pos
      self.color = color
      self.startHealth = float(self.health)
      self.points = self.health/3
      self.explodeSize = sqrt(self.health) * 70
      self.vel = Vector(0, -self.speed, 0)
      self.fireOffset = self.radius
      self.init()
      self.weaponVel = self.weapons[0].projectileSpeed
   
   def die(self):
      Ship.die(self)
      points = int(self.points * player.players.damageFactor)
      if isinstance(self.lastHitBy, Player):
         self.lastHitBy.score += int(points)
      else:
         player.players.distributePoints(points)
            
      if random() < powerupFreq:
         powerups.newPowerup(self.pos)
   
   def fire(self, target):
      dAngle = fixAngle(self.orientation - self.targetAngle)
      if abs(dAngle) < self.fireAngle:
         for w in self.weapons:
            w.fire(self.pos+Vectorp(self.fireOffset, self.orientation), self.orientation, [target])

   def damage(self, a):
       Ship.damage(self, a)
       self.color.b = (1 - (self.health / self.startHealth))/2
   
   def update(self):
      Ship.update(self)
      if player.players.areAlive():
         self.fire(player.players.getClosest(self.pos))
         
   def steer(self, ships):
      ships.remove(self)
      if ships:
         dst, closest = min([(dist(self.pos, x.pos), x) for x in ships])
         if not isinstance(closest, Player) and dst < 2*(self.radius + closest.radius):
            self.avoid(closest)
         elif isinstance(closest, Player) and dst < 4*(self.radius+closest.radius):
            self.avoid(closest)
         else:
            p = player.players.getClosest(self.pos)
            if p:
               self.seek(p)
      self.vel.dir = self.orientation


class Turret(AI, Object):
   radius = 2
   orientation = 0
   explodeSize = 300
   turnSpeed = 0.03
   fireAngle = 0.5
   fireSideAngle = 1.6
   target = None
   zOffset = 1
   
   def __init__(self, ship, weapon, side=None):
      self.model = getModel('turret%d' % randint(0,2))
      self.weapon = weapon
      self.weaponVel = weapon.projectileSpeed
      self.ship = ship
      self.color = ship.color
      self.side = side
      self.update()
      
   def update(self):
      self.move()
      if self.target:
         self.seek(self.target)
         
   def move(self):
      s = self.side
      o = self.ship.orientation
      z = self.ship.zxorientation
      r = self.ship.radius-2
      self.offset = Vector([0,0,0])
      self.offset.x = r*cos(o+s) - r*abs(sin(s))*(1-cos(z))*cos(o+s)
      self.offset.y = r*sin(o+s) - r*abs(sin(s))*(1-cos(z))*sin(o+s)
      self.offset.z = r*sin(s)*sin(z) + self.zOffset
      self.pos = self.ship.pos + self.offset
      self.vel = self.ship.vel
   
   def fire(self, target):
      self.target = target
      o = self.orientation
      if self.side and not abs(o-(self.side+self.ship.orientation)) < self.fireSideAngle:
         return
      dAngle = fixAngle(o - self.targetAngle)
      if abs(dAngle) < self.fireAngle:
         self.weapon.fire(self.pos+Vectorp(self.radius, o), o, [target])
   
   def turn(self, a):
      self.orientation += a
   
#   def draw(self):
#      p = self.pos
#      l = 20
#      if self.target:
#         a = self.getTargetAngle()
#         l = dist(self.pos, self.target.pos)
#         drawLine(p, p+Vectorp(l, a), Color(0,1,.3))
#      o = self.orientation
#      drawLine(p, p+Vectorp(l, o), Color(1,0,.5))
#      Object.draw(self)
   

class CenterTurret(Turret): 
   def move(self):
      self.pos = self.ship.pos + Vector([0,0,self.zOffset])
      self.vel = self.ship.vel
   

class BattleShip(Enemy):
   model = getModel('battleship')
   maxRotation = 1
   radius = 9.5
   health = 220
   turnSpeed = .006
   speed = 0.22
   preEffects = [tilt]
   
   def init(self):
      n = choice([3,5])
      self.weapons = [choice(weapon.battleShipWeapons)(self, slow=True) for x in range(n)]
      self.turrets = [Turret(self, w, a*(2*pi/n)) for a,w in zip(range(n), self.weapons)]
      self.explodeables = self.turrets

   def update(self):
      for t in self.turrets:
         t.update()
      Enemy.update(self)

   def draw(self):
      Ship.draw(self)
      if self.alive:
         for t in self.turrets:
            t.draw()
   
   def fire(self, target):
      for t in self.turrets:
         t.fire(target)
   
   
class Saucer(BattleShip):
   model = getModel('saucer')
   radius = 12
   health = 270
   turnSpeed = .004
   speed = 0.17
   
   def init(self):
      n = randint(3,8)
      self.weapons = [choice(weapon.battleShipWeapons)(self, slow=True) for x in range(n)]
      self.turrets = [Turret(self, w, a*(2*pi/(n-1))) for a,w in zip(range(n-1), self.weapons)]
      ct = CenterTurret(self, self.weapons[-1])
      ct.zOffset = 2
      self.turrets.append(ct)
      self.explodeables = self.turrets
      
class MiniSaucer(BattleShip):
   model = getModel('minisaucer')
   radius = 4
   health = 50
   turnSpeed = .015
   speed = 0.3
   preEffects = [tilt, effects.Thruster(lambda self: 3)]
   
   def init(self):
      self.weapons = [choice(weapon.battleShipWeapons)(self, slow=True)]
      self.turrets = [CenterTurret(self, self.weapons[0])]
      self.explodeables = self.turrets


class Teapot(Enemy):
   model = getModel('teapot')
   turnSpeed = .01
   speed = 0.1
   radius = 50
   health = 2000
   fireAngle = 2*pi
   preEffects = []
   fireOffset = radius - 10
   def init(self):
      self.weapons = [weapon.InsaneRapidScramCannon(self), weapon.DoubleMissile(self)]
   def steer(self, otherShips):
      pass
   def fire(self, target):
      a = getTargetAngle(self.pos, self.weaponVel, target)
      for w in self.weapons:
         w.fire(self.pos+Vectorp(self.fireOffset, a), a, [target])

class TriStar(Enemy):
   turnSpeed = .03
   speed = 0.4
   radius = 3
   health = 20
   model = getModel('ship1')
   preEffects = [tilt, effects.Thruster(lambda self: 2)]
   def init(self):
      self.weapons = [weapon.SeekerGun(self, slow=True)]

class VShip(Enemy):
   turnSpeed = .02
   speed = 0.4
   radius = 3
   health = 40
   model = getModel('ship2')
   def init(self):
      self.weapons = [weapon.Machinegun(self, slow=True)]

class SphereShip(Enemy):
   turnSpeed = .04
   speed = 0.25
   radius = 7.5
   health = 60
   model = getModel('sphere')
   def init(self):
      self.weapons = [weapon.Helixgun(self, slow=True)]

class XShip(Enemy):
   turnSpeed = .01
   speed = 0.3
   radius = 3
   health = 30
   model = getModel('xship')
   def init(self):
      self.weapons = [weapon.beamweapon.WeakLasergun(self)]
      self.fireOffset = 0

class Swarm(list):
   smallShips = [TriStar, VShip, SphereShip, XShip, MiniSaucer]
   largeShips = [BattleShip, Saucer]
   allShips = smallShips + largeShips
   giantShips = [Teapot]
   area = 0
   
   def remove(self, ship):
      for w in ship.weapons:
         self.lostProjectiles += w.projectiles
      del self[self.index(ship)]
   
   ships = property(lambda self: list(self))
   
   def init(self):
      self.lostProjectiles = weapon.lostProjectiles()
      self.newShipTimer = timer.Timer(2500)
   
   def newShip(self):
      if self.area < 4000:
         availableShips = self.smallShips
      elif self.area < 6000:
         self.newShipTimer = timer.Timer(2100)
         availableShips = self.allShips
      elif self.area < 17000:
         self.newShipTimer = timer.Timer(1700)
         availableShips = self.largeShips
      else:
         availableShips = self.giantShips
         self.maxShips = 1
      w, h, z = display.screenSize
      self.append(choice(availableShips)(Vector([uniform(-w, w), h+9, uniform(-5, 5)]),
          cEnemyRand()))
      self.area = w*h

   def update(self):
      if self.newShipTimer.ready():
         self.newShip()
      for ship in self:
         if not ship.alive or not ship.onScreen():
            self.remove(ship)
      for ship in self:
         ship.steer(self.ships + player.players.ships)
         ship.update()
      self.lostProjectiles.update()
         
   def getCollision(self):
      for ship in self:
         for w in ship.weapons:
            w.getCollision(player.players.ships+self.ships)
      otherShips = self.ships
      for ship in self:
         otherShips.remove(ship)
         for oship in otherShips:
            if isCollision(ship, oship):
               collideShips(ship, oship)
      self.lostProjectiles.getCollision(player.players.ships+self)
               
   def draw(self):
      for ship in self:
         ship.draw()
      self.lostProjectiles.draw()

enemies = Swarm()


# vim: set et sts=3 sw=3:
