#!/usr/bin/python

try:
    import psyco
    psyco.full()
except ImportError:
   pass

from util import *
import effects
import timer
from display import display
import sound
import player
import options
from gravityship import player
from solarsystem import *
   
YELLOW = Color(1,1,0)
BLUE = Color(0,0,1)
GREEN = Color(0,1,0)
RED = Color(1,0,0)
BLUR = False
pygame.init()


class Game: 
   paused = False
   maxfps = 50.0 # this is not very exact
   msecperf = 1000/maxfps
   splashLocation = Vector(-25, 15, 25)
   
   def __init__(self):
      display.setCameraZ(1000)
      display.gravityLightInit()
#      sound.reloadMusic()
   
   def update(self):
      effects.update()
      player.update()
      bodies.update()
      if player.alive:
         display.cameraPos.x = display.cameraLookAt.x = player.pos.x
         display.cameraPos.y = display.cameraLookAt.y = player.pos.y

   def getCollisions(self):
      bodies.getCollision(player)
#      player.getCollision(enemy.enemies)
        
   def draw(self):
      display.clearAll()
      display.objectCamera()
      display.screenSize = display.getScreenSize()
      if options.background:
         effects.stars.draw()
      
      bodies.draw()
      if display.cameraPos.z <= 10E2:
         player.draw()
         effects.explosion.draw()
         
      display.hudCamera()
      drawLine(Vector(-5,0,0), Vector(5,0,0), RED)
      drawLine(Vector(0,5,0), Vector(0,-5,0), RED)
      if player.alive:
         mousePos = self.getMousePos()
         player.orientation = a = getAngle(player.pos, mousePos)
         d = dist(player.pos, mousePos)
         drawLine(player.pos + Vectorp(d/2, a), player.pos + Vectorp(d+2, a), GREEN, .5)
         drawLine(mousePos+Vectorp(-2, a+pi/2), mousePos+Vectorp(2, a+pi/2), GREEN, .5)      
         if display.cameraPos.z >= 10E2:
            player.drawTriangle()
         
      display.wordCamera()
      self.drawWords()
      pygame.display.flip()

   def drawWords(self):
      x, y, z = display.screenSize
#      effects.drawWord(Vector(-45, -33, 10), str(int(player.score)), YELLOW)
      effects.drawWord(Vector(35, -33, 10), str(int(player.health)), GREEN)
      if options.fps:
         effects.drawWord(Vector(-45, 30, 10), str(int(timer.clock.get_fps())).zfill(2), BLUE)
      if self.paused:
         effects.drawWord(self.splashLocation, "PAUSED", BLUE)
    
   def getMousePos(self):
      w, h = display.screenResolution
      x, y = pygame.mouse.get_pos()
      a, b, z = gluProject(0, 0, 0)
      return Vector(gluUnProject(x, h-y, z))
   
   def handleMouse(self):
      left, middle, right = pygame.mouse.get_pressed()
      if left:
         player.fire([])
    
   def handleKeys(self):
      keystate = pygame.key.get_pressed()
      player.acc = Vector(0,0,0)
      player.thrusting = False
      if keystate[K_UP] or keystate[K_w]:
         player.acc = Vectorp(player.accrate, player.orientation)
         player.thrusting = True
      if keystate[K_1]:
         display.setCameraZ(200)
      if keystate[K_2]:
         display.setCameraZ(1000)
      if keystate[K_3]:
         display.setCameraZ(5000)
      if keystate[K_4]:
         display.setCameraZ(25000)
      if keystate[K_5]:
         display.setCameraZ(125000)
      if keystate[K_6]:
         display.setCameraZ(625000)

      if keystate[K_p]:
         self.paused = True
         pygame.time.wait(100)
         pygame.mixer.music.pause()
         while self.paused:
            pygame.time.wait(20)
            self.handleEvents()
            keystate = pygame.key.get_pressed()
            timer.tick()
            self.draw()
            self.paused = not (keystate[K_p] or keystate[K_SPACE] or keystate[K_ESCAPE])
         pygame.mixer.music.unpause()
         pygame.time.wait(100)
      if keystate[K_ESCAPE]:
        if player.alive:
           player.die()           
      
   def handleEvents(self):
      for event in pygame.event.get():
         if event.type == QUIT:
            sys.exit()
         elif event.type == VIDEORESIZE:
            display.reshape(event.w, event.h)
         elif event.type == sound.ENDMUSIC:
            sound.reloadMusic()
      while not pygame.display.get_active():
         pygame.time.wait(100)
         pygame.event.pump()
         timer.clock.tick()
         
   def doTimingStuff(self):
      timer.tick()
      lastStep = timer.clock.get_time()
      if lastStep < self.msecperf and options.limitFramerate:
         pygame.time.wait(int(round(self.msecperf - lastStep)))
     
   def run(self):
      while 1: #player.alive:
         self.handleKeys()
         self.handleMouse()
         self.handleEvents()
         self.update()
         self.getCollisions()
         self.draw()
         self.doTimingStuff()
      pygame.mixer.music.fadeout(1)
#      EndSplash(hiscore.write(player.score)).run()
   
    
if __name__ == '__main__':
   Game().run()

# vim: set et sts=3 sw=3:
