from util import *
import weapon
import sound
import effects
import player

class PowerUp(Object):
   noise = sound.getSound('shipupgrade')
   radius = 4
   explodeSize = 500
   vel = Vector([0, -.5, 0])
   alpha = 0.5
   
   def __init__(self, pos):
      self.preEffects = [Spinner(uniform(2,6), randomAxis())]
      self.pos = pos
      self.orientation = uniform(0, 2*pi)
   
   @staticmethod
   def isAvailable():
      return True
   
        
class HealthPowerUp(PowerUp):
   model = getModel('health')
   color = Color(0.2, 1.0, 0.0)
   amount = 75
   def apply(self, ship):
      if sound.isSound:
         self.noise.play()
      ship.health += self.amount
       

class WeaponPowerUp(PowerUp):
   noise = sound.getSound('weaponupgrade')
   model = getModel('rocket')
   color = Color(1, 0.2, 0)
   def apply(self, ship):
      weapon.upgradeWeapon(ship)
      if sound.isSound:
         self.noise.play()
   @staticmethod
   def isAvailable():
      return player.players.areWeaponsUpgradeable()
            
class ShipPowerUp(PowerUp):
   model = getModel('shield')
   color = Color(0,.2,1)
   def apply(self, ship):
      if ship.modelID >= 3:
         return
      ship.modelID += 1
      ship.model = getModel('warthog%d' % ship.modelID)
      for e in ship.postEffects:
         if isinstance(e, WireModel):
            e.model = WireModel.getWireModel(ship.model)
      ship.maxHealth = 100 + 20*ship.modelID
      ship.health += 20
      if sound.isSound:
         self.noise.play()
   @staticmethod
   def isAvailable():
      return player.players.areShipsUpgradeable()

allPowerups = [WeaponPowerUp, HealthPowerUp, ShipPowerUp]

def init():
   global powerups
   powerups = []

def newPowerup(pos):
   availablePowerups = []
   for powerup in allPowerups:
      if powerup.isAvailable():
         availablePowerups.append(powerup)
   powerups.append(choice(availablePowerups)(pos))

def update():
   for powerup in powerups:
      powerup.move()
      if not powerup.onScreen():
         powerups.remove(powerup)

def getCollision(ship):
   for powerup in powerups:
      if isCollision(powerup, ship):
         powerup.apply(ship)
         powerup.vel = Vector(0,0,0)
         effects.explosion.shrinkModel(powerup)
         powerups.remove(powerup)

def draw():
   for powerup in powerups:
      powerup.draw()
   

# vim: set et sts=3 sw=3:
