from util import *
import effects
import sound

class Projectile(Object):
   radius = 1
   alpha = 0.7
   hitNoiseList = [sound.getSound('hit1'), sound.getSound('hit2'),
                    sound.getSound('hit3'), sound.getSound('hit4')]
   
   def __init__(self, pos, vel, color):
      self.pos = pos
      self.vel = vel
      self.orientation = vel.dir
      self.color = color
      
   def hit(self, ship):
      dmg = self.getDamage(ship)
      ship.damage(dmg)
      effects.explosion.explodeParticles(sqrt(dmg)*100, self.pos, Vector([0,0,0]), 0, self.color)
      self.die()
      if sound.isSound:
         choice(self.hitNoiseList).play()
   
   def getDamage(self, ship):
      return self.damage #* cDiff(self.color, ship.color)
   
   def turn(self, a):
      self.orientation += a

class BulletProto(Projectile):
   speed = 2
   damage = 6
   def __init__(self, pos, dir, color):
      Projectile.__init__(self, pos, Vectorp(self.speed, dir), color)
      self.init()
   def init(self):
      pass

class Star(BulletProto):
   model = getModel('star')
   damage = 5
   def init(self):
      self.preEffects = [Spinner(7, (0, 1, 0))]
   
class Triangle(BulletProto):
   damage = 6.5
   model = getModel('triangle')
   def init(self):
      self.preEffects = []

class Tetrahedron(BulletProto):
   model = getModel('tetrahedron')
   def init(self):
      self.preEffects = [Spinner(10, randomAxis())]

BulletTypes = [Triangle, Star, Tetrahedron]
def bulletMaker():
   return choice(BulletTypes)

def slowBulletMaker(speedFactor):
   w = choice(BulletTypes)
   def slowBullet(*args):
      a = w(*args)
      a.vel *= speedFactor
      return a
   return slowBullet

def bombFragmentMaker():
    w = choice(BulletTypes)
    def fragment(*args):
        a = w(*args)
        a.damage /= 2
        a.vel /=2
        return a
    return fragment

class Missile(Accelerator, Projectile):
   damage = 7
   model = getModel('missile')
   preEffects = [Spinner(3, (0,1,0))]
   def __init__(self, pos, vel, acc, color):
      Projectile.__init__(self, pos, vel, color)
      self.acc = acc
      self.orientation = acc.dir
   def getSpeed(self, target):
      avgvel = self.getAvgVel(target)
      print '%s get avg vel to target %d' % (self, avgvel)
      return avgvel

class rotateAroundCenterer(Spinner):
   def __init__(self, spinSpeed, spinAxis, spin, amp):
      Spinner.__init__(self, spinSpeed, spinAxis, degrees(spin))
      self.amp = amp
   def __call__(self, obj):
      Spinner.__call__(self, obj)
      s = radians(self.spin)
      obj.pos = obj.centerPos + Vectorp(self.amp*sin(s), obj.orientation+pi/2)
      obj.pos.z = self.amp*cos(s)

class Spiral(Projectile):
   damage = 10
   model = getModel('leaf')
   speed = 1.0
   def __init__(self, pos, dir, color, amp=3, spin=0):
      Projectile.__init__(self, pos, Vectorp(self.speed, dir), color)
      self.centerPos = pos.copy
      self.preEffects = [rotateAroundCenterer(10, (0, 1, 0), spin, amp)]
   def move(self):
      self.centerPos += self.vel*timer.percTicked
      

class SeekerMissile(Projectile, AI):
   damage = 10
   explodeSize = 300
   model = getModel('missile2')
   lifeTime = 7000
   preEffects = [effects.Thruster(), tilt]
   speed = 1.0
   zxorientation = 0
   weaponVel = speed
   
   def __init__(self, pos, o, color, accuracy, targets):
       Projectile.__init__(self, pos, Vectorp(self.speed, o), color)
       self.targets = targets
       self.target = closest(self.pos, targets)
       self.turnSpeed = accuracy
       self.lifeTime = timer.Timer(self.lifeTime)
   
   def checkTarget(self):
      if self.lifeTime.ready():
         effects.explosion.explodeModel(self)
         self.die()
      if not self.target or not self.target.alive:
         self.target = closest(self.pos, self.targets)
      else:
         self.seek(self.target)
      
   def update(self):
      self.checkTarget()
      self.vel.dir = self.orientation
   
   def turn(self, a):
      self.orientation += a
      self.zxorientation -= 3*a
 
 
class ScramShot(Accelerator, SeekerMissile):
   damage = 2
   explodeSize = 300
   zxorientation = 0
   speed = 0.5
   accrate = 0.04
   weaponAcc = accrate
   scale = 0.6
   preEffects = []
   weaponVel = property(Accelerator.getAvgVel)
   
   def __init__(self, *args):
      SeekerMissile.__init__(self, *args)
      self.orientation = uniform(-pi, pi)
      self.acc = Vectorp(self.accrate, self.orientation)
   
   def update(self):
      self.checkTarget()
      if 0.1 < self.lifeTime.percReady() < 0.16:
          self.vel.dir = self.orientation
      else:
         self.acc.dir = self.orientation
 
class Flame(effects.ExpiringParticle, Projectile):
   damage = 5
   model = getModel('triangle')
   def __init__(self, pos, vel, color):
      effects.ExpiringParticle.__init__(self, pos, vel, color, 1500)
      self.orientation = uniform(0, pi/2)
   die = Projectile.die
   def getDamage(self, ship):
      return self.damage * cDiff(self.color, ship.color) * self.alpha

      
class Bomb(Projectile):
   model = getModel('sphere')
   scale = .2
   speed = .7
   explodeNoise = sound.getSound('boom')
   
   def __init__(self, pos, dir, color, weapon, targets, subBullets):
      Projectile.__init__(self, pos, Vectorp(self.speed, dir), color)
      self.weapon = weapon
      self.targets = targets
      self.subBullets = subBullets
      self.Bullet = bombFragmentMaker()
   
   def update(self):
      for t in self.targets:
         d = dist(self.pos, t.pos) - self.radius - t.radius
         ang = getAngle(self.pos, t.pos)
         a = fixAngle(self.orientation - getTargetAngle(self.pos, self.vel.magAlong(ang), t))
         if (abs(a) < pi/4 and d < 5) or (abs(a) > pi/4 and d < 15):
            self.explode()
            break
   
   def explode(self):
      if sound.isSound:
         self.explodeNoise.play()
      effects.explosion.explodeParticles(300, self.pos, Vector([0,0,0]), self.radius, self.color)
      for i in range(self.subBullets):
         b = self.Bullet(self.pos.copy, uniform(-pi, pi), self.color)
         self.weapon.append(b)
      self.die()
   
   def hit(self, ship):
      self.explode()


# vim: set et sts=3 sw=3:
