from pygame import time
from math import *

lastUpdate = 0
percTicked = 0
clock = time.Clock()
msecPerStep = 20.0
turns = 0

def tick():
   global lastUpdate, percTicked, turns
   percTicked = clock.tick() / msecPerStep
   lastUpdate = time.get_ticks()
   turns+=1

class nTurnTimer:
   def __init__(self, turns):
      self.period = turns
   def ready(self):
      return not turns % self.period

tenTurnTimer = nTurnTimer(10)
fiveTurnTimer = nTurnTimer(5)
twoTurnTimer = nTurnTimer(2)

class Oscillator:
   def __init__(self, period, amplitude=1):
      self.period = float(period)
      self.amplitude = amplitude
      self.start = lastUpdate
   def value(self):
      return self.amplitude * sin(2*pi*((lastUpdate-self.start)/self.period))


class Timer:
   """a timer class, will return True for ready() atleast every msec milliseconds"""
   def __init__(self, length, stopped=False):
      self.last = lastUpdate
      self.length = float(length)
      self.stopped = stopped
   def percReady(self):
      return (lastUpdate - self.last) / self.length
   def ready(self):
      if self.stopped:
         return False
      elif self.last + self.length <= lastUpdate:
         self.last = lastUpdate
         return True
      return False
   def start(self):
      self.stopped = False
      self.last = lastUpdate
   def stop(self):
      self.stopped = True


# vim: set et sts=3 sw=3:
