from random import random, uniform, gauss, randint, choice
from math import *
import os.path
import sys
import weakref
from copy import copy

try:
   from OpenGL.GL import *
   from OpenGL.GLU import *
   from OpenGL.GLUT import *
except ImportError:
   print 'pyOpenGl is not installed.'
   sys.exit(1)

try:
   import pygame
   from pygame.locals import *
except ImportError:
   print 'PyGame is not installed. http://pygame.org/'
   sys.exit(1)

from color import *
from ai import *
from utilfuncs import *
import wavefront
import timer
from display import display

LoadedTextures = weakref.WeakValueDictionary()

def loadTexture(name):
    filename = os.path.join('textures', '%s.png' % name)
    textureSurface = pygame.image.load(filename)
    textureData = pygame.image.tostring(textureSurface, "RGBA", True)
    texid = glGenTextures(1)
    glBindTexture(GL_TEXTURE_2D, texid)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(),
                  textureSurface.get_height(), 0, 
                  GL_RGBA, GL_UNSIGNED_BYTE, textureData)
    return texid


LoadedModels = weakref.WeakValueDictionary()

def getModel(name):
   if name not in LoadedModels:
      model = wavefront.load(os.path.join('models', '%s.obj' % name), name)
      LoadedModels[name] = model
   return LoadedModels[name]

class Object(object):
   alive = True
   alpha = 1.0
   scale = 1.0
   preEffects = []
   postEffects = []

   def draw(self):
      glPushMatrix()
      glTranslate(*self.pos)
      glRotate(degrees(self.orientation - pi/2), 0, 0, 1.0)
      glScale(self.scale, self.scale, self.scale)
      r, g, b = self.color
      glColor4d(r, g, b, self.alpha)
      for pre in self.preEffects:
         pre(self)
      self.drawModel()
      for post in self.postEffects:
         post(self)
      glPopMatrix()

   def drawModel(self):
#      glEnableClientState(GL_VERTEX_ARRAY)
#      glVertexPointerf(self.model.vertices)
##      glNormalPointerf(self.model.normals)
#      glDrawElementsui(GL_POLYGON, [[v for v,n in f] for f in self.model.faces])
#      glDisableClientState(GL_VERTEX_ARRAY)      
    for face in self.model.faces:
         glBegin(GL_POLYGON)
         for v, n in face:
            glNormal3dv(self.model.normals[n])
            glVertex3dv(self.model.vertices[v])
         glEnd()
   
   def update(self):
      pass
  
   def move(self):
      self.pos += self.vel*timer.percTicked
   
#   def onScreen(self):
#      x, y, z = gluProject(*self.pos)
#      w, h = display.screenResolution()
#      return x<w+10 or y<h+10
   
   def onScreen(self):
      return display.onScreen(self.pos)
      
   def die(self):
      assert self.alive, 'Can not die if we are aleady dead'
      self.alive = False
      

### Effects classes and helper functions

def randomAxis():
   return [random(), random(), random()]

class Spinner:
   def __init__(self, spinSpeed, spinAxis, spin=0):
      self.spinSpeed = spinSpeed
      self.spinAxis = spinAxis
      self.spin = spin
   def __call__(self, obj):
      self.spin += self.spinSpeed*timer.percTicked
      glRotate(self.spin, *self.spinAxis)

def incline(obj):
    glRotate(degrees(obj.zyorientation), 1, 0, 0)

def tilt(obj):
    glRotate(degrees(obj.zxorientation), 0, 1, 0)


class WireModel:
   def __init__(self, model):
      self.model = self.getWireModel(model)
   
   def __call__(self, obj):
      r, g, b = ~obj.color
      glColor(r, g, b, obj.alpha/2.0)
      self.drawWireModel()
      
   def drawWireModel(self):
      for face in self.model.faces:
         glBegin(GL_LINE_LOOP)
         for v, n in face:
            glNormal3fv(self.model.normals[n])
            glVertex3fv(self.model.vertices[v])
         glEnd()
   
   @staticmethod
   def getWireModel(model):
      wireModel = wavefront.Object(copy(model.vertices), 
                                    copy(model.normals), 
                                    copy(model.faces), 
                                    model.name)
      for face in model.faces:
         for v, n in face:
            x, y, z = model.vertices[v]
            a = atan2(y, x)
            za = atan2(z, sqrt(x**2+y**2))
            wireModel.vertices[v] = [x + .5*cos(a), y + .5*sin(a), z + .5*sin(za)]
      return wireModel


### Classes that should be inherited by enemies or projectiles

class Accelerator:
   #acc
   def getAvgVel(self):
      a = getAngle(self.pos, self.target.pos)
      return avgVel(self.vel.magAlong(a), self.acc.magAlong(a), 
                     dist(self.pos, self.target.pos), self.target)
   def move(self):
      self.vel += self.acc*timer.percTicked
      self.pos += self.vel*timer.percTicked
      
      


# vim: set et sts=3 sw=3:
