from util import *
import effects
import sound
from projectile import *
import beamweapon
from weaponbase import *

class lostProjectiles(Weapon):
   def __init__(self):
      self.ship = None


class BulletWeapon(Weapon):
   def __init__(self, ship, slow=False):
      self.projectileSpeed = 2
      if slow or self.type == 'side':
         speedFactor = .7
         self.Bullet = slowBulletMaker(speedFactor)
         self.projectileSpeed *= speedFactor
         self.shotNoise = sound.getSound('plasma', 0.6)
      else:
         self.Bullet = bulletMaker()
         self.shotNoise = sound.getSound('pistolshot', 0.3)
      Weapon.__init__(self, ship, slow)

class Machinegun(BulletWeapon):
   type = 'center'
   level = 0
   reload = 275
   def getProjectiles(self, pos, angle, color, targets):
      return [self.Bullet(pos, angle, color)]

class FastMachinegun(Machinegun):
   level = 1
   reload = 150

class DoubleMachinegun(Machinegun):
   level = 1
   reload = 300
   angle = .05
   def getProjectiles(self, pos, o, color, targets):
      return [self.Bullet(pos + Vectorp(2, o+pi/2), o-self.angle, color),
              self.Bullet(pos + Vectorp(2, o-pi/2), o+self.angle, color)]

class WideMachinegun(DoubleMachinegun):
   type = 'side'
   level = 0
   reload = 300
   def init(self):
      self.angle = uniform(.1, pi/4)

class MissileLauncher(Weapon):
   shotNoise = sound.getSound('missile1')
   type = 'center'
   level = 1
   reload = 200
   projectileSpeed = 5
   def getProjectiles(self, pos, o, color, targets):
       if len(targets) > 0:
           self.ship.weaponVel = avgVel(.25, .045, dist(self.ship.pos,
           targets[0].pos), targets[0])
           return [Missile(pos, Vectorp(.25, o),
               Vectorp(uniform(.03, .06), o), color)]
       return []
             
class DoubleMissile(MissileLauncher):
   level = 2
   reload = 250
   angleFunc = lambda s: gauss(.1, .3)
   def getProjectiles(self, pos, o, color, targets):
#      self.ship.weaponVel = avgVel(0, .25, dist(self.ship.pos,
#                                    targets[0].pos), targets[0])
      return [Missile(pos + Vectorp(2, o+pi/2),
              Vectorp(.25, o+self.angleFunc()),
              Vectorp(uniform(.03, .06), o), color),
          Missile(pos + Vectorp(2, o-pi/2),
              Vectorp(.25, o-self.angleFunc()),
              Vectorp(uniform(.03, .06), o), color)
          ]

class WideMissile(DoubleMissile):
   type = 'side'
   level = 1
   reload = 250
   angleFunc = lambda s: uniform(pi/4, pi/2)

class SpiralGun(Weapon):
   shotNoise = sound.getSound('laser4')
   type = 'center'
   level = 1
   reload = 250
   projectileSpeed = Spiral.speed
   def getProjectiles(self, pos, o, color, targets):
      return [Spiral(pos, o, color, 2)]

class WideSpiralGun(SpiralGun):
   level = 1
   reload = 200
   projectileSpeed = Spiral.speed
   def getProjectiles(self, pos, o, color, targets):
      return [Spiral(pos, o, color, 10)]

class Helixgun(SpiralGun):
   level = 1
   reload = 500
   def getProjectiles(self, pos, o, color, targets):
      return [Spiral(pos, o, color), Spiral(pos, o, color, spin=pi)]
  
class SeekerGun(Weapon):
   shotNoise = sound.getSound('missile1')
   reload = 750
   type = 'seek'
   level = 0
   accuracy = .03
   projectileSpeed = SeekerMissile.speed
   def getProjectiles(self, pos, o, color, targets):
     return [SeekerMissile(pos, o, color, self.accuracy, targets)]
  
class ScramCannon(SeekerGun):
   shotNoise = sound.getSound('laser3')
   reload = 1750
   type = 'seek'
   level = 1
   accuracy = 0.09
   projectilesPerShot = 10
   def getProjectiles(self, pos, o, color, targets):
     return [ScramShot(pos.jitter(1.0), o+gauss(0, .2), color, self.accuracy, targets) for i in range(self.projectilesPerShot)]

class RapidScramCannon(ScramCannon):
   reload = 250
   level = 2
   projectilesPerShot = 2

class InsaneRapidScramCannon(ScramCannon):
   reload = 200
   level = 3
   projectilesPerShot = 5

class Flamethrower(Weapon):
   shotNoiseList = [sound.getSound('flame0'), sound.getSound('flame1'),
                     sound.getSound('flame2')]
   reload = 30
   type = 'center'
   level = 1
   projectileSpeed = .7
   def getProjectiles(self, pos, o, color, targets):
      self.shotNoise = choice(self.shotNoiseList)
      return [Flame(pos, Vectorp(self.projectileSpeed, o+gauss(0, .3)), color)]

class BombGun(Weapon):
   shotNoise = sound.getSound('cannon')
   reload = 800
   type = 'center'
   level = 1
   projectileSpeed = Bomb.speed
   def getProjectiles(self, pos, o, color, targets):
      return [Bomb(pos, o, color, self, targets, 25)]

class BigBombGun(BombGun):
   reload = 400
   level = 2
   def getProjectiles(self, pos, o, color, targets):
      return [Bomb(pos, o, color, self, targets, 35)]
   
class SideBombGun(BigBombGun):
   reload = 500
   level = 2
   type = 'side'
   def getProjectiles(self, pos, o, color, targets):
      return [Bomb(pos.copy, o+pi/6, color, self, targets, 35),
               Bomb(pos.copy, o-pi/6, color, self, targets, 35)]
   

maxWeaponLevel = 3

playerWeapons = [Machinegun, FastMachinegun, DoubleMachinegun,
                  WideMachinegun, DoubleMissile, WideMissile,
                  Helixgun, Flamethrower, MissileLauncher,
                  SeekerGun, SpiralGun, WideSpiralGun,
                  ScramCannon, RapidScramCannon, InsaneRapidScramCannon,
                  BombGun, BigBombGun, SideBombGun] + beamweapon.playerWeapons
                  
battleShipWeapons = [SeekerGun, MissileLauncher, DoubleMachinegun, FastMachinegun,
                      ScramCannon, RapidScramCannon, InsaneRapidScramCannon,
                      SpiralGun]


def getAptWeapons(lvl, type):
   aptWeapons = []
   for w in playerWeapons:
      if w.type == type and w.level == lvl:
         aptWeapons.append(w)
   return aptWeapons

def newWeapon(level=0):
   return choice(getAptWeapons(level, 'center'))

def areWeaponsUpgradeable(ship):
   if len(ship.weapons) < maxWeaponLevel:
      return True
   for w in ship.weapons:
      if w.type == 'seek':
         break
   else:
      return True
   for w in ship.weapons:
      posUpgrades = getAptWeapons(w.level+1, w.type)
      if posUpgrades:
         return True

def upgradeWeapon(ship):
   if len(ship.weapons) < 2:
      ship.weapons.append(choice(getAptWeapons(0, 'side'))(ship))
      return True
   for w in ship.weapons:
      if w.type == 'seek':
         break
   else:
      ship.weapons.append(choice(getAptWeapons(0, 'seek'))(ship))
   for w in ship.weapons:
      posUpgrades = getAptWeapons(w.level+1, w.type)
      if posUpgrades:
         ship.lostProjectiles += w.projectiles
         ship.weapons.remove(w)
         ship.weapons.append(choice(posUpgrades)(ship))
         return True


# vim: set et sts=3 sw=3:
